/* -*-C++-*- */
/*
 * MTBDD representation.
 *
 * Copyright (C) 2004 Carnegie Mellon University
 *
 * This file is part of Tempastic-DTP.
 *
 * Tempastic-DTP is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * Tempastic-DTP is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Tempastic-DTP; if not, write to the Free Software
 * Foundation, Inc., #59 Temple Place, Suite 330, Boston, MA
 * 02111-1307 USA
 *
 * $Id: mtbdd.h,v 2.2 2005/02/08 11:36:41 lorens Exp $
 */
#ifndef MTBDD_H
#define MTBDD_H

#include <config.h>
#include "problems.h"
#include "events.h"
#include "formulas.h"
#include "expressions.h"
#include "phfit.h"
#include <map>
#include <string>
#include <vector>
#include <util.h>
#include <cudd.h>


/* ====================================================================== */
/* PHEventMap */

/*
 * A mapping from event names to phase-type parameters.
 */
struct PHEventMap : public std::map<std::string, PHParameters> {
};


/* ====================================================================== */
/* Policy */

/*
 * A policy.
 */
struct Policy {
  /* Constructs a policy. */
  Policy(DdManager* dd_man, DdNode* mapping,
	 const std::vector<const Event*>& actions,
	 const PHEventMap& ph_parameters,
	 const std::map<const Event*, size_t>& phase_index)
    : dd_man_(dd_man), mapping_(mapping), actions_(actions),
      ph_parameters_(ph_parameters), phase_index_(phase_index) {}

  /* Deletes this policy. */
  ~Policy();

  /* Returns the phase-type parameters. */
  const PHEventMap& ph_parameters() const { return ph_parameters_; }

  /* Returns a phase index. */
  const std::map<const Event*, size_t>& phase_index() const {
    return phase_index_;
  }

  /* Returns the action selected for the given state. */
  const Event* action(const AtomSet& atoms, const ValueMap& values,
		      const std::vector<size_t>& phases) const;

private:
  /* DD Manager. */
  DdManager* dd_man_;
  /* An MTBDD mapping states to action ids. */
  DdNode* mapping_;
  /* The actions of this policy. */
  std::vector<const Event*> actions_;
  /* Phase-type parameters. */
  PHEventMap ph_parameters_;
  /* A phase index. */
  std::map<const Event*, size_t> phase_index_;
};


/* ====================================================================== */
/* mtbdd_solve */

/* Partition options. */
enum PartitionOption { FULL_PARTITION, EVENT_PARTITION, NO_PARTITION,
		       UNIFORMIZATION };

/* Filter options. */
enum FilterOption { NO_FILTER, H_FILTER, T_FILTER, A_FILTER };

/* Solves the given problem using MTBDDs. */
void mtbdd_solve(DdManager*& dd_man, const Problem& problem,
		 double gamma, double epsilon,
		 PartitionOption partition, FilterOption filter,
		 int moments, int phases, size_t sample_size);


#endif /* MTBDD_H */
